//////////////////////////////////////////////////////////////////////////////////////////
// Project description
// 
// Name: Touch Tracer
// Purpose: General test application for detected blobs
//
// Author: Laurence Muller (aka Falcon4ever)
// Contact: info@falcon4ever.com
// Site: www.lauw.biz (blog) / www.falcon4ever.com (misc)
// Licensetype: GNU General Public License (GPL)
//
// Read main.cpp for project details
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
// Include files
// 
#include "touch_tracer.h"
int font = (int)(intptr_t)GLUT_BITMAP_8_BY_13;

//////////////////////////////////////////////////////////////////////////////////////////
// Main class
// 

// Init Touch Tracer class
Touch_Tracer::Touch_Tracer()
{	
	c[0].R = 1.0f; c[0].G = 0.0f; c[0].B = 0.0f;	// Red
	c[1].R = 0.0f; c[1].G = 1.0f; c[1].B = 0.0f;	// Green
	c[2].R = 0.0f; c[2].G = 0.0f; c[2].B = 1.0f;	// Blue		
	c[3].R = 1.0f; c[3].G = 0.0f; c[3].B = 1.0f;	// Magenta
	c[4].R = 1.0f; c[4].G = 1.0f; c[4].B = 0.0f;	// Yellow
	c[5].R = 0.0f; c[5].G = 1.0f; c[5].B = 1.0f;	// Cyan
	c[6].R = 1.0f; c[6].G = 0.5f; c[6].B = 1.0f;	// Orange

	current_colour = 0;	
}

// Destroy Touch Tracer class
Touch_Tracer::~Touch_Tracer()
{
	// empty	
}

// Notify that a finger has just been made active. 
void Touch_Tracer::fingerDown(TouchData data)
{
	// add to list
	trace[data.ID][1].X = data.X;
	trace[data.ID][1].Y = data.Y;
	trace[data.ID][1].area = data.area;
	
	blob_colour[data.ID] = current_colour;

	current_colour++;
	if(current_colour > 6)
		current_colour = 0;
}

// Notify that a finger has moved 
void Touch_Tracer::fingerUpdate(TouchData data)
{	
	map<int, Point>::iterator iter2;	

	iter2 = --trace[data.ID].end();
	trace[data.ID][(*iter2).first+1].X = data.X;
	trace[data.ID][(*iter2).first+1].Y = data.Y;
	trace[data.ID][(*iter2).first+1].area = data.area;

	// Erase first state of current blobs if list is larger than 20
	if((int)trace[data.ID].size() > 20)
	{
		iter2 = trace[data.ID].begin();
		trace[data.ID].erase((*iter2).first);
	}
}

// A finger is no longer active..
void Touch_Tracer::fingerUp(TouchData data)
{
	blob_colour.erase(data.ID);
	trace.erase(data.ID);	
}

void Touch_Tracer::draw_blobs()
{
	if(trace.size() > 0)
	{
		// Paint trace
		map<int, map<int, Point> >::iterator iter1;		
		map<int, Point>::iterator iter2;	
		
		// Run through blobs
		float alpha;				
		for(iter1 = trace.begin(); iter1 != trace.end(); iter1++)
		{	
			// Fat tail
			glLineWidth(5.5);
			alpha = 0.0;
			glBegin(GL_LINE_STRIP);					
			for(iter2 = (*iter1).second.begin(); iter2 != (*iter1).second.end(); iter2++)
			{
				// Use the colour lookup table...
				glColor4f(c[blob_colour[(*iter1).first]].R, 
						  c[blob_colour[(*iter1).first]].G, 
						  c[blob_colour[(*iter1).first]].B, 
						  alpha);	

				glVertex2f(	trace[(*iter1).first][(*iter2).first].X*screen_width, 
							trace[(*iter1).first][(*iter2).first].Y*screen_height);					
				alpha += 0.1;
			}		
			glEnd();				

			// Create a circle (better figure out some faster/better way?)
			glLineWidth(2.0);
			iter2 = --(*iter1).second.end();
			
			const int SEGMENTS = 12;
			const float DEG2RAD = 2 * 3.14159 / SEGMENTS;
			const float RADIUS = 15.0;
			
			glBegin(GL_LINE_LOOP);
 			for (int i = 0; i < 12; i++)
			{
				float degInRad = i * DEG2RAD;
				glVertex2f(iter2->second.X*screen_width + cos(degInRad)*RADIUS, iter2->second.Y*screen_height + sin(degInRad)*RADIUS);  // wasteful to recalculate sin/cos
			} 
			glEnd();		

			// Draw blob info
			glLineWidth (1.0);
			draw_info((*iter1).first, iter2->second.X, iter2->second.Y, iter2->second.area);
		}		
	}
	debug_info();
}

void Touch_Tracer::renderBitmapCharacter(int x, int y, void *font, char *string)
{			
	char *c;
	glRasterPos2i(x, y);
	
	for (c=string; *c != '\0'; c++) 
		glutBitmapCharacter(font, *c);					
}

void Touch_Tracer::draw_info(int ID, float X, float Y, float Area)
{		
		glColor3f(1.0f, 0.0f, 0.0f);
		glBegin(GL_LINES);			
			glVertex2f((X*screen_width)		, (Y*screen_height)	);
			glVertex2f((X*screen_width) + 16, (Y*screen_height) + 16);
			glVertex2f((X*screen_width) + 16, (Y*screen_height) + 16);
			glVertex2f((X*screen_width) + 16, (Y*screen_height) + 52);
		glEnd( );		
				
		char buffer[64];
		sprintf(buffer, "ID #%d (%d)", ID, (int)Area);   
		renderBitmapCharacter((int)(X*screen_width)+17, (int)(Y*screen_height)+26, (void *)(intptr_t)font, buffer);
		sprintf(buffer, "X = %d", (int)(X*screen_width));   
		renderBitmapCharacter((int)(X*screen_width)+17, (int)(Y*screen_height)+26+12, (void *)(intptr_t)font, buffer);
		sprintf(buffer, "Y = %d", (int)(Y*screen_height));   
		renderBitmapCharacter((int)(X*screen_width)+17, (int)(Y*screen_height)+26+12+12, (void *)(intptr_t)font, buffer); 
}

void Touch_Tracer::debug_info()
{	
	char buffer[64];	
	static int frame=0, time, timebase=0;
	static float currentfps;

	++frame;
	time = glutGet(GLUT_ELAPSED_TIME);

	if (time - timebase > 1000) 
	{
		// printf("FPS:%4.2f\n", frame*1000.0/(time-timebase));
		currentfps = frame * 1000.0 / (time - timebase);
		timebase = time;		
		frame = 0;
	}

	glColor4f(1.0, 1.0, 1.0, 1.0);
	sprintf(buffer, "Touch tracer v%s by Laurence Muller", VERSION);   // dont change please :)
	renderBitmapCharacter(10, 24, (void *)(intptr_t)font, buffer);
	sprintf(buffer, "Average FPS: %4.2f", currentfps);   
 	renderBitmapCharacter(10, 40, (void *)(intptr_t)font, buffer);
	sprintf(buffer, "Display settings: %dx%d %db @ %dHz", screen_width, screen_height, bitdepth, refreshrate);   	
	renderBitmapCharacter(10, 56, (void *)(intptr_t)font, buffer);						
	sprintf(buffer, "Amount of blobs: %d", trace.size());   
	renderBitmapCharacter(10, screen_height-16, (void *)(intptr_t)font, buffer);			
}

void Touch_Tracer::set_screenproperties(int w, int h, int bd, int rr)
{
	screen_width = w;
	screen_height = h;
	bitdepth = bd;
	refreshrate = rr;
}

void Touch_Tracer::reset_blobs()
{		
	if(trace.size() > 0)
	{		
		current_colour = 0;	
		blob_colour.clear();

		// Both methods dont seem to work, no idea why :/
		/* Quick way to clean <map>.
		map<int, map<int, Point>>::iterator iter1;							
		for(iter1 = trace.begin(); trace.size() > 0;)
		{
			iter1 = trace.begin();
			trace.erase((int)(*iter1).first);
		}
		*/

		// trace.clear();	// Not working? :(
	}	
}
