//////////////////////////////////////////////////////////////////////////////////////////
// Project description
// 
// Name: Touch Tracer
// Purpose: General test application for detected blobs
//
// Author: Laurence Muller (aka Falcon4ever)
// Contact: info@falcon4ever.com
// Site: www.lauw.biz (blog) / www.falcon4ever.com (misc)
// Licensetype: GNU General Public License (GPL)
//
// Read main.cpp for project details
//////////////////////////////////////////////////////////////////////////////////////////

// Version settings
#define VERSION "0.2"
#define RELDAY "08"
#define RELMONTH "05"
#define RELYEAR "2007"

#ifdef WIN32
	#define _CRT_SECURE_NO_DEPRECATE	
#endif

#include <map>
#include <vector>

// Touchlib (set path)
#include "TouchScreenDevice.h"
#include "TouchData.h"
#ifdef WIN32
	#pragma comment(lib, "touchlib.lib")
#endif

// OpenCV
#include <cv.h>
#include <highgui.h>
#ifdef WIN32
	#pragma comment(lib, "cv.lib")
	#pragma comment(lib, "cxcore.lib")
	#pragma comment(lib, "highgui.lib")
#endif

// Glut/OpenGL
#if defined(WIN32)
	#include <glut.h>
#elif defined(__APPLE__)
	#include <GLUT/glut.h>
#else
	#include <GL/glut.h>
#endif

// Blob/Point structure
struct Point
{
	float X;
	float Y;
	float area;
};

// RGB color struct (blobs)
struct s_RGB
{
	float R;
	float G;
	float B;
};

// Used namespaces
using namespace std;
using namespace touchlib;

// Touch_tracer class
class Touch_Tracer : public ITouchListener
{
	public:
		Touch_Tracer();
		~Touch_Tracer();

		virtual void fingerDown(TouchData data);
		virtual void fingerUpdate(TouchData data);
		virtual void fingerUp(TouchData data);

		void draw_blobs();		
		void set_screenproperties(int w, int h, int bd, int rr);
		void reset_blobs();		

	private:		
		void draw_info(int ID, float X, float Y, float Area);
		void renderBitmapCharacter(int x, int y, void *font, char *string);
		void debug_info();

		map<int, map<int, Point> > trace;
		int screen_width, screen_height, bitdepth, refreshrate;

		int current_colour;
		s_RGB c[7];
		map<int, int> blob_colour;
};

